;====================================================
; Auto-Scroll hack
; By Erik557
;====================================================
; Rewrites sprite F3 (Auto scroll) to allow:
; - Autoscroll in vertical levels
; - Custom autoscroll speeds
; - Autoscroll that goes to the left
;
; Requires 2 bytes of Free RAM.
;====================================================
; NOTE: In vertical levels, the speed will depend on
; the X position of the scroll and NOT on the Y pos.
;====================================================

	!dp = $0000
	!addr = $0000
	!sa1 = 0
	!gsu = 0

if read1($00FFD6) == $15
	sfxrom
	!dp = $6000
	!addr = !dp
	!gsu = 1
elseif read1($00FFD5) == $23
	sa1rom
	!dp = $3000
	!addr = $6000
	!sa1 = 1
endif

!AllowHrzScroll    = 1             ; set to 0 to not allow the screen to scroll horizontally
                                   ; in vertical levels (lm's option is always overriden...)

!RAM_ScrollSprYPos = $18B4|!addr   ; both should be from $0100 to $1FFF
!RAM_ScrollSprDir  = $18BB|!addr

if !sa1
org read3($02A885)
       autoclean JSL SetUpScrollRAM
       RTL
else
org $02A884
       autoclean JML SetUpScrollRAM
endif

org $05C787
       BRA +
       NOP #4
+

org $05C793
       autoclean JML ScrollHack
freecode

SetUpScrollRAM:
       LSR #2                      ;\ save y position to free RAM
       STA.w !RAM_ScrollSprYPos    ;/
       LDA $1440|!addr             ;\
       AND #$01                    ; | save direction to free RAM
       STA.w !RAM_ScrollSprDir     ;/
if !sa1
       SEP #$10                    ;\
       JSL $05BCD6                 ; | restore
       REP #$10                    ;/
       RTL                         ; return
else
       JSL $05BCD6                 ; restore
       JML $02A888                 ; return
endif

ScrollHack:
       PHB : PHK : PLB             ; set data bank

       LDA.w !RAM_ScrollSprYPos    ;\  'restore'
       ASL                         ; | use the y position as a speed index
       TAY                         ;/

       LDA.w !RAM_ScrollSprDir
       ASL
       STA $55
       STA $56
       
       LDX $1456|!addr             ; layer to process

       LDA $5B                     ;\
       AND #$01                    ; | check for vertical levels
       BEQ .horizontal             ;/

.vertical
       if !AllowHrzScroll == 1
              LDA #$01                    ;\ allow for horizontal scrolling
              STA $1411|!addr                   ;/ (lm setting is, for some reason, overriden)
       endif

       REP #$20
       LDA $1448|!addr,x           ; load layer 1 y speed
       CMP Yspeeds,y               ; >tfw no cmp $000000,y
       BEQ ..notMove               ; if already the same, don't accelerate
       LDY $55
       BEQ +
       INC                         ; increase x-pos of the screen
       BRA ..notMove
+      DEC                         ; decrease x-pos of the screen
..notMove
       STA $1448|!addr,x

       LDY $55                     ;\ if scrolling downwards, return
       BNE ..noStop                ;/
       LDA $1C                     ;\
       CMP #$0004                  ; | don't make the camera go fuck itself
       BCS ..noStop                ; |
       STZ $1448|!addr,x           ;/

..noStop
       LDA $5F
       DEC
       XBA
       AND #$FF00
       CMP $1464|!addr,x
       BNE +
       STZ $1448|!addr,x

       ; the following routine doesn't exist for y positions
       ; so i have to write my own
+      LDA $1450|!addr,x
       AND #$00FF
       CLC : ADC $1448|!addr,x
       STA $1450|!addr,x
       AND #$FF00
       BPL +
       ORA #$00FF
+      XBA
       CLC : ADC $1464|!addr,x
       STA $1464|!addr,x
       LDA $08
       EOR #$FFFF
       INC
       STA $08

       LDY $55                     ;\ don't check if going upwards
       BEQ ..return                ;/
       LDA $80                     ; wtf nintendo why wasn't there a hurt routine
       CMP #$FFC0                  ; if not too far up the screen, return
       BNE ..return                ; bcc and bcs didn't help so i went insane and used bne
       SEP #$20
       JSL $00F60A                 ; kill mario
       STZ $7D                     ; don't make mario fall
..return
       PLB : JML $05C32B           ; return

.horizontal
       REP #$20
       LDA $1446|!addr,x           ; load layer 1 x speed
       CMP Xspeeds,y               ; >tfw no cmp $000000,y
       BEQ ..notMove               ; if already the same, don't accelerate
       LDY $55
       BEQ +
       INC                         ; increase x-pos of the screen
       BRA ..notMove
+      DEC                         ; decrease x-pos of the screen
..notMove
       STA $1446|!addr,x
       LDY $55                     ;\ if scrolling to the right, return
       BNE ..return                ;/
       LDA $1A                     ;\
       CMP #$0004                  ; | don't make the camera go fuck itself
       BCS ..return                ; |
       STZ $1446|!addr,x           ;/
..return
       PLB : JML $05C7A7           ; return

Xspeeds:
       dw $FF20,$01FF,$0010,$FF20,$0010,$0000,$0000,$0000      ; maximum scrolling speed, you can theoretically set this to anything.
       dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000      ; don't go overboard though.


Yspeeds:
       dw $0200,$01FF,$0010,$FF20,$0010,$0000,$0000,$0000      ; maximum scrolling speed, you can theoretically set this to anything.
       dw $0000,$0000,$0000,$0000,$0000,$0000,$0000,$0000      ; don't go overboard though.

